==> search_api_elasticsearchkit_proxy-1.0.1.tar.gz: search_api_elasticsearchkit_proxy/src/Controller/ElasticSearchkitProxyController.php <==
<?php

namespace Drupal\search_api_elasticsearchkit_proxy\Controller;

use Drupal\elasticsearch_connector\Entity\Cluster;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Controller\ControllerBase;
use Elasticsearch\ClientBuilder;
use Drupal\search_api\Entity\Server;

/**
 * Class ElasticsearchProxyController.
 *
 * A controller for proxying Elasticsearch search requests through Drupal.
 * This class is designed to be extensible and customizable by other modules.
 */
class ElasticSearchkitProxyController extends ControllerBase {

  /**
   * Handles the Elasticsearch search request.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object containing the Elasticsearch query.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The response object containing the Elasticsearch response or error.
   */
  public function search(Request $request) {
    $server = $this->loadSearchApiServer();

    if (!$server) {
      return $this->handleServerNotFound();
    }

    $client = $this->initializeElasticsearchClient($server);

    try {
      $body = $request->getContent();
      $response = $this->performElasticsearchQuery($client, $body);
      return $this->handleSuccessResponse($response);
    }
    catch (\Exception $e) {
      return $this->handleErrorResponse($e);
    }
  }

  /**
   * Loads the active search API server connector.
   *
   * @return \Drupal\search_api\ServerInterface|null
   *   The search API server or null if not found.
   */
  protected function loadSearchApiServer() {
    $config = $this->config('gg_searchkit.settings');
    $server_id = $config->get('search_api_server');
    return Server::load($server_id);
  }

  /**
   * Handles the case when the search API server is not found.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The error response.
   */
  protected function handleServerNotFound() {
    return new JsonResponse(['error' => 'Search API server not found.'], 404);
  }

  /**
   * Initializes the Elasticsearch client.
   *
   * @param \Drupal\search_api\ServerInterface $server
   *   The search API server.
   *
   * @return \Elasticsearch\Client
   *   The Elasticsearch client.
   */
  protected function initializeElasticsearchClient($server) {
    $cluster = $this->loadElasticsearchCluster($server);
    $clusterOptions = $cluster->get('options');

    $host = $cluster->get('url');
    $clientBuilder = ClientBuilder::create()->setHosts([$host]);

    if ($this->isBasicAuthentication($clusterOptions)) {
      $username = $clusterOptions['username'];
      $password = $clusterOptions['password'];
      $clientBuilder->setBasicAuthentication($username, $password);
    }

    return $clientBuilder->build();
  }

  /**
   * Loads the Elasticsearch cluster entity.
   *
   * @param \Drupal\search_api\ServerInterface $server
   *   The search API server.
   *
   * @return \Drupal\elasticsearch_connector\Entity\Cluster
   *   The Elasticsearch cluster entity.
   */
  protected function loadElasticsearchCluster($server) {
    $clusterName = $server->get('backend_config')['cluster_settings']['cluster'];
    return Cluster::load($clusterName);
  }

  /**
   * Checks if the cluster uses Basic Authentication.
   *
   * @param array $clusterOptions
   *   The cluster options.
   *
   * @return bool
   *   TRUE if Basic Authentication is used, FALSE otherwise.
   */
  protected function isBasicAuthentication(array $clusterOptions) {
    return $clusterOptions['authentication_type'] === 'Basic'
      && isset($clusterOptions['username'])
      && isset($clusterOptions['password']);
  }

  /**
   * Performs the Elasticsearch query.
   *
   * @param \Elasticsearch\Client $client
   *   The Elasticsearch client.
   * @param string $body
   *   The request body containing the Elasticsearch query.
   *
   * @return array
   *   The Elasticsearch response.
   */
  protected function performElasticsearchQuery($client, $body) {
    return $client->msearch(['body' => $body]);
  }

  /**
   * Handles a successful Elasticsearch response.
   *
   * @param array $response
   *   The Elasticsearch response.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response containing the Elasticsearch results.
   */
  protected function handleSuccessResponse(array $response) {
    $jsonResponse = new JsonResponse($response);
    $jsonResponse->headers->set('Access-Control-Allow-Origin', '*');
    $jsonResponse->headers->set('Access-Control-Allow-Methods', 'GET, POST, OPTIONS');
    $jsonResponse->headers->set('Access-Control-Allow-Headers', 'Content-Type, Authorization');

    return $jsonResponse;
  }

  /**
   * Handles an error during the Elasticsearch request.
   *
   * @param \Exception $e
   *   The exception encountered during the request.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The error response containing the exception message.
   */
  protected function handleErrorResponse(\Exception $e) {
    $errorResponse = new JsonResponse(['error' => $e->getMessage()], 500);
    $errorResponse->headers->set('Access-Control-Allow-Origin', '*');
    $errorResponse->headers->set('Access-Control-Allow-Methods', 'GET, POST, OPTIONS');
    $errorResponse->headers->set('Access-Control-Allow-Headers', 'Content-Type, Authorization');

    return $errorResponse;
  }

}

==> search_api_elasticsearchkit_proxy-2.0.0.tar.gz: search_api_elasticsearchkit_proxy/src/Controller/ElasticSearchkitProxyController.php <==
<?php

namespace Drupal\search_api_elasticsearchkit_proxy\Controller;

use Drupal\elasticsearch_connector\Plugin\search_api\backend\ElasticSearchBackend;
use Drupal\search_api\ServerInterface;
use Elastic\Elasticsearch\Client;
use Elastic\Elasticsearch\Response\Elasticsearch;
use Http\Promise\Promise;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Core\Controller\ControllerBase;
use Drupal\search_api\Entity\Server;

/**
 * Class ElasticsearchProxyController.
 *
 * A controller for proxying Elasticsearch search requests through Drupal.
 * This class is designed to be extensible and customisable by other modules.
 */
class ElasticSearchkitProxyController extends ControllerBase {

  /**
   * Initialize the client.
   *
   * @throws \Drupal\search_api\SearchApiException
   */
  private function initialize(): JsonResponse|Client {
    $server = $this->loadSearchApiServer();

    if (!$server instanceof ServerInterface) {
      return $this->handleServerNotFound();
    }

    return $this->initializeElasticsearchClient($server);

  }

  /**
   * Loads the active search API server connector.
   *
   * @return \Drupal\search_api\ServerInterface|null
   *   The search API server or null if not found.
   */
  protected function loadSearchApiServer(): ?ServerInterface {
    $config = $this->config('search_api_elasticsearchkit_proxy.settings');
    $server_id = $config->get('search_api_server');
    return Server::load($server_id);
  }

  /**
   * Handles the case when the search API server is not found.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The error response.
   */
  protected function handleServerNotFound(): JsonResponse {
    return new JsonResponse(['error' => 'Search API server not found.'], 404);
  }

  /**
   * Initializes the Elasticsearch client.
   *
   * @param \Drupal\search_api\ServerInterface $server
   *   The search API server.
   *
   * @return \Elastic\Elasticsearch\Client
   *   The Elasticsearch client.
   *
   * @throws \Drupal\search_api\SearchApiException
   */
  protected function initializeElasticsearchClient(ServerInterface $server): Client {
    $serverBackend = $server->getBackend();
    if (!$serverBackend instanceof ElasticSearchBackend) {
      throw new \RuntimeException('Cannot connect to a Search API server that is not an ElasticSearchBackend.');
    }

    return $serverBackend->getClient();
  }

  /**
   * Handles the Elasticsearch search request.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object containing the Elasticsearch query.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The response object containing the Elasticsearch response or error.
   */
  public function search(Request $request): JsonResponse {
    $client = $this->initialize();

    try {
      $body = $request->getContent();
      $response = $this->performElasticsearchQuery($client, $body);
      return $this->handleSuccessResponse($response->asArray());
    }
    catch (\Exception $e) {
      return $this->handleErrorResponse($e);
    }
  }

  /**
   * Handles the Elasticsearch msearch request.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object containing the Elasticsearch query.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The response object containing the Elasticsearch response or error.
   */
  public function msearch(Request $request): JsonResponse {
    $client = $this->initialize();

    try {
      $body = $request->getContent();
      $response = $this->performElasticsearchQuery($client, $body, 'msearch');
      return $this->handleSuccessResponse($response->asArray());
    }
    catch (\Exception $e) {
      return $this->handleErrorResponse($e);
    }
  }

  /**
   * Performs the Elasticsearch query.
   *
   * @param \Elastic\Elasticsearch\Client $client
   *   The Elasticsearch client.
   * @param string $body
   *   The request body containing the Elasticsearch query.
   * @param string|null $type
   *   The type of search.
   *
   * @return \Elastic\Elasticsearch\Response\Elasticsearch|\Http\Promise\Promise
   *   The Elasticsearch response.
   */
  protected function performElasticsearchQuery(Client $client, string $body, ?string $type = NULL): Elasticsearch|Promise {
    return match ($type) {
      'msearch' => $client->msearch(['body' => $body]),
      default => $client->search(['body' => $body]),
    };
  }

  /**
   * Handles a successful Elasticsearch response.
   *
   * @param array $response
   *   The Elasticsearch response.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The JSON response containing the Elasticsearch results.
   */
  protected function handleSuccessResponse(array $response): JsonResponse {
    $jsonResponse = new JsonResponse($response);
    $jsonResponse->headers->set('Access-Control-Allow-Origin', '*');
    $jsonResponse->headers->set('Access-Control-Allow-Methods', 'GET, POST, OPTIONS');
    $jsonResponse->headers->set('Access-Control-Allow-Headers', 'Content-Type, Authorization');

    return $jsonResponse;
  }

  /**
   * Handles an error during the Elasticsearch request.
   *
   * @param \Exception $e
   *   The exception encountered during the request.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   The error response containing the exception message.
   */
  protected function handleErrorResponse(\Exception $e): JsonResponse {
    $errorResponse = new JsonResponse(['error' => $e->getMessage()], 500);
    $errorResponse->headers->set('Access-Control-Allow-Origin', '*');
    $errorResponse->headers->set('Access-Control-Allow-Methods', 'GET, POST, OPTIONS');
    $errorResponse->headers->set('Access-Control-Allow-Headers', 'Content-Type, Authorization');

    return $errorResponse;
  }

}
